using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace PFCTour
{
	/// <summary>
	/// PFMain - main form.
	/// </summary>
	public class PFMain : System.Windows.Forms.Form
	{
		#region Variables

		private System.ComponentModel.IContainer components;

		Graphics gfx;   // Draws to the backbuffer.
		Graphics formgfx;  // Draws to the form.

		Bitmap backbuffer;  // Empty bitmap.
		Bitmap playerbmp;  // This will hold the main character graphics.
		// Actually, it is a bunch of frames of the character.
		Rectangle playerloc;
		// This will hold the current location of the player.  X and Y included.
		// Why didn't I use Point?  Because I am probably going to use the height and width for collisions later.
		bool goleft, goright, gofast;
		// Indicates if the left or right buttons are pressed.

		int chardirec;
		// Which direction is the character facing?
		int animcycler;
		// This is the animation cycler that will count from 0 to 3 and back to 0.
		bool isjumping;
		// While this is true, the character will be jumping in the air.
		int playerveloc;
		/* This is the velocity of the player in the Y direction.
		 * Positive values point down.
		 * To jump up, playerveloc must be initialized to a negative value. */
		bool godown;
		// Indicates if the down arrow is pressed.
		int leftend, rightend;
		// These are the left and right ends of the cliff that the character is currently on.
		ArrayList platforms;
		// Holds the list of all of the current platforms.
		Rectangle coin;
		// The coin location on the form.
		Bitmap coinbmp;
		// The picture of the coin.
		int anim;
		// The animation counter for the coins.
		Rectangle wall;
		// The location of the wall.

		private System.Windows.Forms.Timer clock;
		#endregion
		#region Constants
		const int themapwidth = 800;
		const int themapheight = 600;
		// I need to determine a default spot for my land.
		// It has to be between 0 and MAPHEIGHT, since that's the height of my backbuffer.
		// Preferably closer to 600 as more land equals less playing area.

		const int thelandheight = 568;
		// 600 - 32 = 568... I like 32. 
		// Constants are good for you.

		const int thecharwidth = 16;
		const int thecharheight = 32;
		// These define the width of the character in the file.
		const int themovespeed = 5;
		// How much I move every tick.
		const int thewalkingframecount = 4;
		// How many frames are in each walking cycle.
		const int thesbarinitialvelocity = -32;
		// The initial velocity for the player's jump using space bar.
		const int thegravity = 4;
		// Gravitational acceleration constant... how much the speed changes at each tick.
		const int thelowjumpvelocity = -20;
		// The initial velocity for the player's low jump.
		const int theshoulder = 17;
		// Looked in paint to determine the shoulder distance from the bottom of the file.
		// If the platform top falls within this range above the player's feet, the player will get on the platform.
		const int thecoinsize = 16;
		// The width and height of a coin rectangle as it should appear on the form.
		const int thecoinframecount = 4;
		// Number of frames in the coin animation.

		#endregion
		public PFMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.clock = new System.Windows.Forms.Timer(this.components);
			// 
			// clock
			// 
			this.clock.Tick += new System.EventHandler(this.clock_Tick);
			// 
			// PFMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Name = "PFMain";
			this.Text = "Platform Tour";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.PFMain_KeyDown);
			this.Load += new System.EventHandler(this.PFMain_Load);
			this.KeyUp += new System.Windows.Forms.KeyEventHandler(this.PFMain_KeyUp);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new PFMain());
		}


		private void PFMain_Load(object sender, System.EventArgs e)
		{
			// Setup clock interval.
			clock.Interval = 50;
			clock.Enabled = true;

			this.Size = new Size(themapwidth + 8, themapheight + 24);

            /* Create the backbuffer.  This will hold the map information... what you actually will see, because I
            am going to draw it to the form in one fell swoop.  Just that I'm going to be drawing onto the backbuffer
            for all of my miscellaneous artworks, and then draw everything that I accumulated onto this backbuffer to the
            form. :)  */
			backbuffer = new Bitmap(themapwidth, themapheight);
			// gfx is aimed at the backbuffer.
			gfx = Graphics.FromImage(backbuffer);
			// formgfx is aimed at the form.
			formgfx = this.CreateGraphics();

			playerbmp = new Bitmap(Application.StartupPath + @"\..\..\sumult.bmp");
			// Loads the image into this bitmap.
			// Given this arrangement, the bitmap must be in the same directory as the solution.
			playerbmp.MakeTransparent(Color.Magenta);
			// Makes all of the magenta in the bitmap transparent.
			playerloc = new Rectangle(100, thelandheight - thecharheight, thecharwidth, thecharheight);
			// Instantiate the player location.

			platforms = new ArrayList(100);
			// Initialize the arraylist to hold up to 100 elements.
			platforms.Add(Rectangle.FromLTRB(50, 415, 200, thelandheight));
			platforms.Add(Rectangle.FromLTRB(100, 465, 225, thelandheight));
			platforms.Add(Rectangle.FromLTRB(150, 515, 250, PFMain.thelandheight));
			// Instantiate the platforms for the level.

			coin = new Rectangle(200, 550, thecoinsize, thecoinsize);
			coinbmp = new Bitmap(Application.StartupPath + @"\..\..\coin.bmp");
			coinbmp.MakeTransparent(Color.Magenta);
			// Create the coin position on the display and get the coin picture from file.

			wall = Rectangle.FromLTRB(225, 415, 275, 465);
			// Initialize the wall coordinates.

			leftend = 0;
			rightend = themapwidth;
			// Range of the ground set as the end points for the character's current platform.




		}

		private void clock_Tick(object sender, System.EventArgs e)
		{
			this.charactermovement();
			this.artwork();
		}

		private void drawonform() 
		{
			formgfx.DrawImage(backbuffer, 0, 0);
			// The backbuffer was drawn on by the gfx.
		}

		private void PFMain_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Left) 
			{
				goleft = true;
			} 
			else if (e.KeyCode == Keys.Right) 
			{
				goright = true;
			}
			else if (e.KeyCode == Keys.ShiftKey) 
			{
				gofast = true;
			}
			else if (e.KeyCode == Keys.Down) 
			{
				godown = true;
			}
			else if (e.KeyCode == Keys.Space) 
			{
				isjumping = true;
				// The jump is initialized.
				if (godown) 
				{
					playerveloc = PFMain.thelowjumpvelocity;
					// Low jump made when down key is pressed.
				} 
				else 
				{
					playerveloc = PFMain.thesbarinitialvelocity;
					// Use space bar initial velocity.
				}
			}
		}

		private void PFMain_KeyUp(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Left) 
			{
				goleft = false;
			} 
			else if (e.KeyCode == Keys.Right) 
			{
				goright = false;
			}
			else if (e.KeyCode == Keys.Down) 
			{
				godown = false;
			}
			else if (e.KeyCode == Keys.ShiftKey) 
			{
				gofast = false;
			}

			if (  !goleft && !goright  ) 
			{
				// If neither key is pressed.
				animcycler = 0;
				// Reset the counter.
			}
		}

		private void charactermovement() 
		{
			if (goleft)
			{
				if (gofast) 
				{
					playerloc.Offset(-themovespeed - (themovespeed >> 1), 0);
					// When shift is pressed, the player runs.
				} 
				else 
				{
					playerloc.Offset(-themovespeed, 0);
					// Move player left.
				}
				if (playerloc.Left < 0) 
				{
					// If the player goes off the edge of the form, then we will move him back to the edge.
					playerloc.Offset(-playerloc.Left, 0);
				} else if (playerloc.IntersectsWith(wall)) {
					playerloc.Offset(wall.Right - playerloc.Left, 0);
					// Move the player back to the right edge of this wall.
				} else if (playerloc.Right < leftend && !isjumping ) {
					isjumping = true;
					// Falling off the edge of a platform.
					playerveloc = 0;
					// Free-fall.
				}
				chardirec = 0;  // Look left.
				// Now update the animation cycler.
				animcycler = (animcycler + 1) % thewalkingframecount;
			} 
			else if (goright) 
			{
				if (gofast) 
				{
					playerloc.Offset(themovespeed + (themovespeed >> 1), 0);
					// When Shift is pressed, the player runs.
				} 
				else 
				{
					playerloc.Offset(themovespeed, 0);
					// Move player right.
				}
				if (playerloc.Right > themapwidth) 
				{
					// If player goes off right edge... move him back.
					playerloc.Offset(themapwidth - playerloc.Right, 0);
				} else if (playerloc.IntersectsWith(wall)) {
					playerloc.Offset(wall.Left - playerloc.Right, 0);
					// Move player back to left edge of wall.
				} else if (playerloc.Left > rightend && !isjumping ) {
					isjumping = true;
					playerveloc = 0;
					// Falling from edge of platform.
				}
				chardirec = 1;  // Look right.
				// Now update the animation cycler.
				animcycler = (animcycler + 1) % thewalkingframecount;
			}

			if (isjumping) 
			{
				playerloc.Offset(0, playerveloc + thegravity >> 1);
				// dD (.Offset) = Vo (playerveloc) + 0.5 * A (thegravity >> 1)
				playerveloc += thegravity;
				// dV = (playerveloc +=) A (thegravity)
				// Moves the character according to its velocity, gravity, and location.
				if (playerloc.Bottom >= PFMain.thelandheight) 
				{
					// check if we are touching ground (another rect advantage).
					isjumping = false;
					// Stop jumping... we've hit the ground.
					leftend = 0; rightend = PFMain.themapwidth;
					// Ground range.
					animcycler = 0;
					// Reset animation cycler that was counting during the jump.
					playerloc.Offset(0, thelandheight - playerloc.Bottom);
					// Move the player so he stands properly on the bottom.
				} else if (playerloc.IntersectsWith(wall)) {
					if (playerveloc > 0 ) 
					{
						// player is falling and should land on the wall.
						isjumping = false;
						leftend = wall.Left; rightend = wall.Right;
						// Set the end points for the wall.
						animcycler = 0;
						playerloc.Offset(0, wall.Top - playerloc.Bottom);
					} 
					else 
					{
						// The player is moving upwards and should hit head against wall.
						playerloc.Offset(0, wall.Bottom - playerloc.Top);
						// Move the player so that his head touches the top of the wall.
						playerveloc = 0;
					}
				} else {
					foreach (Rectangle platform in platforms) 
					{
						if (playerveloc > 0 && playerloc.Bottom >= platform.Top && playerloc.Bottom - theshoulder <= platform.Top) 
						{
							// The player has landed on the platform.  Using a rectangle for the player made this check much easier than without.
							// We do need to set a few things to let the player know where he is.
							if (playerloc.Right >= platform.Left && platform.Right >= playerloc.Left) 
							{
								// Player is over the platform.
								isjumping = false;
								// Not jumping
								leftend = platform.Left; rightend = platform.Right;
								// Platform endpoints are now the falling endpoints.
								animcycler = 0;
								// Restart the animation cycle to the first frame.
								playerloc.Offset(0, platform.Top - playerloc.Bottom);
								// Position the player so that he is standing on the platform.
							}
						}
					}
				}

			}

			if (coin.IntersectsWith(playerloc)) 
			{
				coin = Rectangle.Empty;
				// set this coin to be an empty rectangle
			}
		}

		private void artwork() 
		{

			anim++;
			// Increment the animation counter.


			/* I am going to draw a lightblue rectangle for the sky.
			It will go down until it reaches the land, so therefore the bottom of the rectangle has to be
			the LANDHEIGHT. */
			gfx.FillRectangle(Brushes.LightSkyBlue, Rectangle.FromLTRB(0, 0, themapwidth, thelandheight));
			// And the remainder of the area is theland, starting from thelandheight.
			gfx.FillRectangle(Brushes.ForestGreen, Rectangle.FromLTRB(0, thelandheight, themapwidth, themapheight));
			// I use LTRB instead of new so that I don't have to calculate the height
			// These two will make the 'background' for the form.

			foreach (Rectangle platform in platforms) 
			{
				gfx.FillRectangle(Brushes.Moccasin , platform);
				gfx.DrawRectangle(Pens.BurlyWood , platform);
				// This draws the platform with a moccasin interior and a burlywood border.
				// Try the colors out to get a good one.
			}

			gfx.FillRectangle(Brushes.Peru, wall);
			gfx.DrawRectangle(Pens.SaddleBrown, wall);
			// Draws the wall onto the form with peruvian interior and saddlebrown border.

			if (isjumping) 
			{
				gfx.DrawImage(playerbmp, playerloc.X, playerloc.Y, new Rectangle((thecharwidth * thewalkingframecount << 1) + chardirec * thecharwidth, 0, thecharwidth, thecharheight), GraphicsUnit.Pixel);
				// Jumping draw only.
			} 
			else 
			{
				gfx.DrawImage(playerbmp, playerloc.X, playerloc.Y , new Rectangle(thecharwidth * chardirec * thewalkingframecount + animcycler * thecharwidth, 0, thecharwidth, thecharheight), GraphicsUnit.Pixel);
				// This draws the character to the display based on the current location.
			}

			if (!coin.IsEmpty) 
			{
				gfx.DrawImage(coinbmp, coin.X, coin.Y, new Rectangle((anim % thecoinframecount) * thecoinsize, 0, thecoinsize, thecoinsize), GraphicsUnit.Pixel);
				// Draw the coin.  Since anim increases, the coin will animation because of the mod and multiply for the left argument.
			}


			this.drawonform();
		}

	}
}
